/** @file
@code
***************************************************************************
    CVS                  : $RCSfile: database.cpp,v $
                           $Revision: 1.1 $
***************************************************************************
@endcode*/
#include "demo_db.h"

Database::Database() {}


Database::~Database() {}


/**
 * Load a database-text file into memory.
 * Multiple files can be loaded.
 * Any duplicate entries are overwritten.
 * @param filename
 * @return
 */
void Database::Load(string filename) {
	int i;
	string line, key, record;
	bool RecordActive=false;
	if (filename=="") {
		throw "Database Load: no filename specified\n";
	};
	//create and open a file-stream object
	ifstream file(filename.c_str());
	if (!file) {
		throw string("Database Load: Unable to open file: "+filename+"\n");
	};
	while ( std::getline(file, line) ) {
		if (line[0]==CONTROL_CHAR) {
			//end processing an active record.
			if (RecordActive) {
				Insert(key, record);
				record = "";
				RecordActive = false;
			};
			//process a "control" line; i.e. a key or comment
			switch (line[1]) {
				case KEY_CHAR			:
					//start a new record.
					// skip whitespace
					i = line.find_first_not_of(" \t\n\r", 2);
					if (i>-1){
						key = line.substr(i);
						record = "";
						RecordActive = true;
					};
					break;
				case COMMENT_CHAR	:
					//ignore line
					break;
				default			:
					//unknown control line: ignore
					break;
			};
		} else {
			if (RecordActive) {
				//return the \n that the getline removes...
				//but only between two lines.
				if (record !="") {
					record += '\n';
				};
				//add line to current record.
				record += line;
			};
		};
	};
	//finish the last open record, if any
	if (RecordActive) {
		//return the \n that the getline removes...
		if (record !="") {
			record += '\n';
		};
		record += line;
		Insert(key,record);
		record = "";
		RecordActive = false;
	};
}

/**
 * Insert one database-item (key + data) into the DB_Map.
 * Expand any escape-strings. Replace "<ESC>" with the escape-code for the PCxx's (\\x1b)
 * When a key already exists its data is replaced with the new data.
 * @param key
 * @param data
 */
void Database::Insert(string key, string data) {
	if (key=="") {
		throw "Database Insert: Cannot insert empty key\n";
	};
	//replace any "<ESC>" with an ESCAPE - ESC == '\x1b'
// 	data = ExpandEsc(data);
	data = ExpandAll(data);
	key = ExpandAll(key);
	if (DB_Map.count(key)==0){
		DB_Map.insert( db_record(key,data) );
	}else {
		DB_Map[key] = data;
	};
#ifdef DEBUG

	cout << "\nDatabase::Insert: key: " << key << "\n with data: \n" << data << endl;
#endif
}

/**
 * Find an item in the database and return its data.
 * Whitespace at begin and end of key is ignored.
 * @param key string containing the key to search for
 * @return string containing data stored with key; empty when key not found.
 */
string Database::Find(string key) {
	int i,j;
	db_iter it;
	//remove whitespace from begin and end
	i = key.find_first_not_of(" \t\n\r");
	j = key.find_last_not_of(" \t\n\r");
	if ((i<0) || (j<0)){
		return "";
	};
	key = key.substr(i, j-i+1);
	it = DB_Map.find(key);
	if (it != DB_Map.end()) {
		return (*it).second;
	} else {
		//not found
#ifdef DEBUG
		cerr << "\n Warning Database::Find: key: " << key << " : NOT found" << endl;
#endif

		return "";
	};
}

/**
 * Clear the Database in memory. The DB_Map is erased.
 */
void Database::Clear() {
	DB_Map.clear();
}

