/** @file
@code
***************************************************************************
  begin                : 
  written by           : 
  CVS                  : $RCSfile: demo_header.h,v $
                         $Revision: 1.1 $
***************************************************************************
@endcode

Common header; contains all includes of standard system headers
*/

#include <iostream>
#include <string>
#include <sstream>
#include <stdlib.h>

#ifdef WIN32
	//disable the (stupid) warning "identifier was truncated in debug information"
	// which happens because of the use of a map with two strings from the STL
	#pragma warning(disable:4786)
	//define bzero as a macro in case it is not provided by the system-libs (windows)
	#ifndef HAVE_BZERO
	#define bzero(ptr,n) memset(ptr,0,n)
	#endif

	#include <windows.h>
	#include <conio.h>
	#define ioctl ioctlsocket
#else
	#include <sys/ioctl.h>
	#include <time.h>
#endif

#include <string.h>

using namespace std;



#define READBUFSIZE 100

//=== constants for main demo_db object

#define def_DBFILE "pc_db.txt"
#define def_PORT 4701
#define def_IP "127.0.0.1"

#define DS175FILENAME "DS175format.txt"
#define PC2FILENAME "PC2format.txt"

#define HELPTEXT "\
Usage: PC_Demo [-fhipv]\n\
\n\
   -f=FILENAME   : set the filename of the database-text file to use.\n\
                    default: pc_db.txt\n\
   -h            : this helptext.\n\
   -i=IPADDRESS  : the IP-address of the pc_ethernet_driver to connect to.\n\
                    In dotted-decimal notation (e.g. 222.222.222.222)\n\
                    default: 127.0.0.1 (localhost)\n\
   -p=PORTNUMBER : the number of the port of the pc_ethernet_driver.\n\
                    default: 4701\n\
   -v            : display version information.\n\
"

#define HELPCMDTEXT "\
Command help:\n\
  - ?, help       : This helptext.\n\
  - quit          : Quit the PriceChecker Database demo.\n\
  - send <file>   : Send the contents of <file> to the PC_Ethernet_Driver.\n\
                    NO formatting is done; the contents of the file is sent\n\
                    as-is, including special characters like LF or CR.\n\
  - loadformat    : (re)Loads the PCxx format files\n\
  - loaddb <name> : Load the database with filename <name>.\n\
                    If no name specified reload the last database.\n\
"

#define VERSIONTEXT "\
-------------------------------\n\
-- PriceChecker PC_Database demo --\n\
--------- Version 1.2 ---------\n\n\
"

#define PC_DEVICENAME "PC_Device"
#define PC_DEVICETYPE1 "DS175"
#define PC_DEVICETYPE1l "ds175"
#define PC_DEVICETYPE2 "PC2"
#define PC_DEVICETYPE2l "pc2"

#define ERRORTEXT01 "Format file for DS175 opened."
#define ERRORTEXT02 "WARNING: DS175 format file ("
#define ERRORTEXT03 "Format file for PC2 opened."
#define ERRORTEXT04 "WARNING: PC2 format file ("

#define INFOTEXT01 "Connected to PC_Ethernet_Driver."

#ifndef DEMO_HEADER_H
#define DEMO_HEADER_H

/**
 * Multi platform Milli-Sleep function.
 * Sleep msec milliseconds.
 * @param msec
 */
inline void mSleep( int msec ) {
#ifdef WIN32
	Sleep( msec );
#endif
#ifndef WIN32

	struct timespec nanotime;
	nanotime.tv_sec = 0;
	nanotime.tv_nsec = msec * 1000000;
	nanosleep( &nanotime, NULL );
#endif
};

#define ESC "\x1b"
#define ESCALIAS "<ESC>"

/**
 * replace any "<ESC>" with an ESCAPE - ESC == '\\x1b'.
 * @param s string containing "<ESC>" substrings.
 * @return a string with all "<ESC>" replaced by the '\\x1b' character.
 */
inline string ExpandEsc( string s ) {
	unsigned int pos;
	string EscStr = ESCALIAS;
	//replace any "<ESC>" with an ESCAPE - ESC == '\x1b'
	pos = 0;
	while ( pos != string::npos ) {
		pos = s.find( EscStr, pos );
		if ( pos != string::npos ) {
			s.replace( pos, EscStr.size(), ESC );
		};
	};
	return s;
}

#define ETX "\x03"
#define ETXALIAS "<ETX>"
/**
 * replace any "<ETX>" with an ETX-char - ETX == '\\x03'.
 * @param s string containing "<ETX>" substrings.
 * @return a string with all "<ETX>" replaced by the '\\x03' character.
 */
inline string ExpandEtx( string s ) {
	unsigned int pos;
	string EtxStr = ETXALIAS;
	//replace any "<ETX>" with an ETX == '\x03'
	pos = 0;
	while ( pos != string::npos ) {
		pos = s.find( EtxStr, pos );
		if ( pos != string::npos ) {
			s.replace( pos, EtxStr.size(), ETX );
		};
	};
	return s;
}

/**
 * replace any single-byte-integer-value between <> with the actual value; <0x05> => \\x05.
 * @param s string containing "<{value}>" substrings.
 * @return a string with all "<{value}>" replaced by the actual value.
 */
inline string ExpandHex( string s ) {
	unsigned int startpos, endpos, val;
	string valstr;
//	char valchar;
	startpos = 0;
	endpos = 0;
	while ( startpos != string::npos ) {
		//find start of the hex-value
		startpos = s.find( "<#", startpos );
		if ( startpos != string::npos ) {
			//find the end of the hex-value
			endpos = s.find( ">", startpos );
			if ( endpos != string::npos ) {
				//extract integer value
				valstr = s.substr( startpos + 2, endpos - startpos - 2 );
				val = strtol( valstr.c_str(), NULL, 0 );
				if ( val < 256 ) {
					//insert integer value in the string
					valstr = (char)val;
					s.replace( startpos, endpos - startpos+1, valstr );
				}
			}
		};
	};
	return s;
}

inline string ExpandAll(string s){
	s = ExpandEsc(s);
	s = ExpandEtx(s);
	s = ExpandHex(s);
	return s;
}

#endif
